	uniform float time;
	uniform float numberWiggles;
	uniform float speed;
	uniform vec3 lightPosition;

	varying vec3 lightVec;
	varying vec3 eyeVec;
	varying vec2 texCoord;
	attribute vec3 tangent;

void main()
{
	// Begin - Wiggle transformation code
	vec4 position = vec4(gl_Vertex);
	float radius;

	// I use a quadratic function rather than a sine function to ensure a smooth taper
	radius = (position.z < 1.0) ? sqrt(position.z / 1.0): sqrt((10.0 - position.z) / 1.0);
	position.xy *= (position.z < 1.0 || position.z > 9.0) ? radius : 1.0;
	position.xy = position.xy * 0.33;
	
	float angle = position.z * (numberWiggles / 2.0) + time * 0.1 * speed;
	
	position.x += sin(angle);

	gl_Position = gl_ModelViewProjectionMatrix * position;
	// End - Wiggle transformation code
	
	
	
	// Begin - Setup Tangent Space
	eyeVec = vec3(gl_ModelViewMatrix * gl_Vertex);
	gl_TexCoord[0] = gl_MultiTexCoord0;

	vec3 N = normalize(gl_NormalMatrix * gl_Normal);
	vec3 T = normalize(gl_NormalMatrix * tangent);
	vec3 B = cross(T, N);
	
	vec3 V;
	
	V.x = -lightPosition.x;
	V.y = -lightPosition.y;
	V.z = -lightPosition.z;
	lightVec = normalize(V);

	V.x = dot(T, eyeVec);
	V.y = dot(B, eyeVec);
	V.z = dot(N, eyeVec);
	eyeVec = normalize(V);	

	texCoord = vec2(gl_MultiTexCoord0);
	texCoord.t = texCoord.t * 4.0;
	// End - Tangent Space
}