/**
*	In object space, perterbs each fragment according to
*	the sampled normal map, and computes diffuse
*	lighting according to the new normal.  Colors each
*	fragment according to the sampled color texture and
*	the light.  See report for full lighting equation.
*/

varying vec4 position;			// IN - the interpolated position
uniform sampler2D colorMap;		// the color texture
uniform sampler2D normalMap;	// the normal map texture

void main( void ) 
{ 
	// normal map from texture, R=x, G=y, B=z
	vec3 NormalTex = texture2D(normalMap, gl_TexCoord[0].st).xyz; 
		NormalTex = (NormalTex - 0.5) * 2.0;
	vec3 normal = normalize(NormalTex);
	
	// computes the newNormal <dot> lightDir for lighting   
	vec3 lightDir = normalize(vec3(gl_LightSource[0].position - position)); 
	float NdotL = max(dot(normal, lightDir), 0.0); 
   
	// computes the fragment color based on the color texture and lighting
	vec4 finalcolor = texture2D(colorMap, gl_TexCoord[0].xy);  
		finalcolor *= gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse * NdotL;
	
	// calculates the ambient and global ambient terms
	vec4 ambient = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
	vec4 globalAmbient = gl_LightModel.ambient * gl_FrontMaterial.ambient;
   
	gl_FragColor = min(finalcolor + ambient + globalAmbient, vec4(1.0)); 
} 