/**
*	project4.cpp
*	CS 6610 - Advanced Computer Graphics I
*	Zach Gildersleeve
*	Nov. 20, 2006
*	
*	Creates a cylinder over a plane that is transformed
*	into a wiggling, bumpmapped snake over a sandy surface,
*	using two GLSL shaders.  One shader (glslGroundBump)
*	bumpmaps and lights the ground - using diffuse lighting
*	and object space normal map - and the other shader
*	(glslWiggle) tapers and wiggles the cylinder, and
*	bumpmaps and lights the snake using a tangent space
*	method and Blinn/Phong half angle model of specular lighting.
*/

#include "./include/main.h"
#include "./include/vector.h"

GLUI				*glui;			// pointers for all of the glui controls
GLUI_Panel			*lightPanel;
GLUI_Panel			*snakePanel;
GLUI_Rollout		*lightRollout;
GLUI_Rollout		*snakeRollout;
GLUI_Spinner		*lightSpin;
GLUI_Spinner		*speedSpin;
GLUI_Spinner		*numberSpin;
GLUI_Translation	*lightTransXY;
GLUI_RadioGroup		*snakeRadio;


int cur_button = -1;					// mouse handling for main window
int last_x;								// mouse position
int last_y;								// mouse position
float eye[3];							// camera position
float lookat[3];						// camera look position
int main_window;						// the main window name
int viewX = 189, viewY, viewW, viewH;	// variables to builds the viewport around the GLUI GUI

GLenum glslGroundBump;					// names of shader program components
GLenum vertexGroundBump;
GLenum fragmentGroundBump;
GLenum vertexWiggle;
GLenum fragmentWiggle;
GLenum glslWiggle;

GLfloat  diffuseLight[] = {1.0, 1.0, 1.0, 1.0};		// color of overall ambient light
GLfloat	 lightPos[]		= {0.0, 5.0, -5.0, 1.0};	// position of light
float lightIntensity	= 1.0;						// controls intensity of scene light
float lightPositionXZ[] = {0.0, -5.0};				// the XY light position

float myTime = 0.0;					// conter for animation
float myNumberWiggles = 1.0;		// the number of wiggles, where one wiggle is a single arch
float mySpeed = 0.3;				// speed of the wiggle animation

static unsigned *image;				// pointer to texture data
static int width;					// width of texture data
static int height;					// height of texture data
static int components;				// components of texture data

GLuint duneTexture;					// names of the texture objects
GLuint sandTexture;
GLuint snakeColorTexture;
GLuint snakeBumpTexture;

int snakeMode = SNAKE_SOLID;		// draws the snake as solid or wireframe

GLuint cylinderList;				// display list for the snake cylinder
GLuint materialList;				// display list for the ground and snake material attributes

GLfloat snakePosition[] = {0.0, 0.0, 0.0};			// position of the snake, used for arrow key movement

/**
*	Idle function for GLUT callback
 */
void myGlutIdle(void)
{
	if (glutGetWindow() != main_window)
		glutSetWindow(main_window);
				myTime += 0.1;

	glutPostRedisplay();
}

/**
*	Mouse callback function for GLUT
 *
 *	@param	button	the current mouse button
 *	@param	state	the state of 'button'
 *	@param	x		the x coord of the mouse
 *	@param	y		the y coord of the mouse
 */
void myGlutMouse(int button, int state, int x, int y)
{
	if (state == GLUT_DOWN)
		cur_button = button;
	else
	{
		if (button == cur_button)
			cur_button = -1;
	}
	
	last_x = x;
	last_y = y;
}

/**
*	Mouse motion callback function for GLUT
 *
 *	@param	x		the x coord of the mouse
 *	@param	y		the y coord of the mouse
 */
void myGlutMotion(int x, int y)
{
	int dx = x-last_x;			// change in mouse position
	int dy = y-last_y;			// change in mouse position
	
	float scale, len, theta;	// controls camera movement
	float neye[3], neye2[3];	// temp eye positions
	float f[3], r[3], u[3];		// more camera coords
	
	switch(cur_button)
	{
		case GLUT_LEFT_BUTTON:	// translate
			f[0] = lookat[0] - eye[0];
			f[1] = lookat[1] - eye[1];
			f[2] = lookat[2] - eye[2];
			u[0] = 0;
			u[1] = 1;
			u[2] = 0;
			
			scale = sqrt(v_getVectorLength(f)) * 0.007;	// scale the change by how far away we are
			
			v_crossProduct(f, u, r);
			v_crossProduct(r, f, u);
			v_normalize(r);
			v_normalize(u);
			
			eye[0] += -r[0]*dx*scale + u[0]*dy*scale;
			eye[1] += -r[1]*dx*scale + u[1]*dy*scale;
			eye[2] += -r[2]*dx*scale + u[2]*dy*scale;
			
			lookat[0] += -r[0]*dx*scale + u[0]*dy*scale;
			lookat[1] += -r[1]*dx*scale + u[1]*dy*scale;
			lookat[2] += -r[2]*dx*scale + u[2]*dy*scale;
			
			break;
			
		case GLUT_MIDDLE_BUTTON:	// zoom
			f[0] = lookat[0] - eye[0];
			f[1] = lookat[1] - eye[1];
			f[2] = lookat[2] - eye[2];
			
			len = v_getVectorLength(f);
			v_normalize(f);
			
			len -= sqrt(len)*dx*0.03;	// scale the change by how far away we are
			
			eye[0] = lookat[0] - len*f[0];
			eye[1] = lookat[1] - len*f[1];
			eye[2] = lookat[2] - len*f[2];
			
			// make sure the eye and lookat points are sufficiently far away
			// push the lookat point forward if it is too close
			if (len < 1)
			{
				lookat[0] = eye[0] + f[0];
				lookat[1] = eye[1] + f[1];
				lookat[2] = eye[2] + f[2];
			}
				break;
			
		case GLUT_RIGHT_BUTTON:	// rotate
			
			neye[0] = eye[0] - lookat[0];
			neye[1] = eye[1] - lookat[1];
			neye[2] = eye[2] - lookat[2];
			
			// first rotate in the x/z plane
			theta = -dx * 0.007;
			neye2[0] = (float)cos(theta)*neye[0] + (float)sin(theta)*neye[2];
			neye2[1] = neye[1];
			neye2[2] =-(float)sin(theta)*neye[0] + (float)cos(theta)*neye[2];
			
			// now rotate vertically
			theta = -dy * 0.007;
			
			f[0] = -neye2[0];
			f[1] = -neye2[1];
			f[2] = -neye2[2];
			u[0] = 0;
			u[1] = 1;
			u[2] = 0;
			v_crossProduct(f, u, r);
			v_crossProduct(r, f, u);
			len = v_getVectorLength(f);
			v_normalize(f);
			v_normalize(u);
			
			neye[0] = len * ((float)cos(theta)*f[0] + (float)sin(theta)*u[0]);
			neye[1] = len * ((float)cos(theta)*f[1] + (float)sin(theta)*u[1]);
			neye[2] = len * ((float)cos(theta)*f[2] + (float)sin(theta)*u[2]);
			
			eye[0] = lookat[0] - neye[0];
			eye[1] = lookat[1] - neye[1];
			eye[2] = lookat[2] - neye[2];
			
			break;
	}
	
	last_x = x;
	last_y = y;
	
	glutPostRedisplay();
}

/**
*	Reshape callback function for GLUT
 *
 *	@param	x	the x window coord - unused
 *	@param	y	the y window coord - unused
 */
void myGlutReshape(int	x, int y)
{
	GLUI_Master.get_viewport_area(&viewX, &viewY, &viewW, &viewH);
	glViewport(viewX, viewY, viewW, viewH);
}

/**
*	Processes the arrow keys, and moves the snake accordingly
*	based on the initial camera and snake positions.
*
*	@param	key	the key pressed
*	@param	x	?
*	@param	y	?
*/
void processArrowKeys(int key, int x, int y)
{
	switch(key)
	{
		case GLUT_KEY_UP:
			snakePosition[0] -= 0.1;
			break;
		case GLUT_KEY_DOWN:
			snakePosition[0] += 0.1;
			break;
		case GLUT_KEY_LEFT:
			snakePosition[2] += 0.1;
			break;
		case GLUT_KEY_RIGHT:
			snakePosition[2] -= 0.1;
			break;
	}
	glutPostRedisplay();
}

/**
*	Loads the source code of a shader program by
*	copying the entire program into a single string.
*
*	@param	filename	the filename and location of the source code
*	@return		a pointer to the built string
*/
const GLcharARB* loadSource(const char* filename) 
{
	std::string buf;
	std::string line;
	
	std::ifstream in(filename);
	
	if(!in) {
		fprintf(stderr, "Could not find file %s\n", filename);
		exit(EXIT_FAILURE);
	}
	
	while(std::getline(in,line)) {
		buf += line; buf += '\n';
	}
	
	GLcharARB* ret = (GLcharARB*)malloc(buf.size()+1);
	strncpy(ret, buf.c_str(), buf.size());
	ret[buf.size()] = '\0';
	
	return ret;
}

/**
*	Creates the GLSL shader programs by loading the source
*	of each object, compiling the shader objects, linking
*	the compiled objects to a GLSL program, and compiling
*	the GLSL program.
*/
void createShaders()
{
	const GLcharARB* vertexGroundBumpSource;
	const GLcharARB* fragmentGroundShadersource;
	const GLcharARB* vertexWiggleSource;
	const GLcharARB* fragmentWiggleSource;
	GLint status;
	
	// Create the OpenGL shader objects
	glslGroundBump    = glCreateProgramObjectARB();
	vertexGroundBump   = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	fragmentGroundBump = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	
	glslWiggle = glCreateProgramObjectARB();
	vertexWiggle = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	fragmentWiggle = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
		
	// Load the source code
	vertexGroundBumpSource = loadSource("./duneBump.vert");
	fragmentGroundShadersource = loadSource("./duneBump.frag");
	vertexWiggleSource = loadSource("./wiggle.vert");
	fragmentWiggleSource = loadSource("./wiggle.frag");
		
	glShaderSourceARB(vertexGroundBump, 1, &vertexGroundBumpSource, NULL);
	glShaderSourceARB(fragmentGroundBump, 1, &fragmentGroundShadersource, NULL);
	glShaderSourceARB(vertexWiggle, 1, &vertexWiggleSource, NULL);
	glShaderSourceARB(fragmentWiggle, 1, &fragmentWiggleSource, NULL);

	delete vertexGroundBumpSource;
	delete fragmentGroundShadersource;
	delete vertexWiggleSource;
	delete fragmentWiggleSource;
	
	// Compile the ground shaders
	glCompileShaderARB(vertexGroundBump);
	glGetObjectParameterivARB(vertexGroundBump, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(vertexGroundBump, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(vertexGroundBump, msglen, NULL, msg);
		fprintf(stderr, "Error compiling vertex shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(fragmentGroundBump);
	glGetObjectParameterivARB(fragmentGroundBump, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(fragmentGroundBump, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(fragmentGroundBump, msglen, NULL, msg);
		fprintf(stderr, "Error compiling fragment shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	
	// Attach the compiled ground shaders to the program object
	glAttachObjectARB(glslGroundBump, vertexGroundBump);
	glAttachObjectARB(glslGroundBump, fragmentGroundBump);
	
	// Link the ground program
	glLinkProgramARB(glslGroundBump);
	glGetObjectParameterivARB(glslGroundBump, GL_OBJECT_LINK_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the linking error
		GLint msglen;
		glGetObjectParameterivARB(glslGroundBump, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(glslGroundBump, 1024, NULL, msg);
		fprintf(stderr, "Error linking GLSL program\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	
	// compile the snake shaders
	glCompileShaderARB(vertexWiggle);
	glGetObjectParameterivARB(vertexWiggle, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(vertexWiggle, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(vertexWiggle, msglen, NULL, msg);
		fprintf(stderr, "Error compiling fragment shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	glCompileShaderARB(fragmentWiggle);
	glGetObjectParameterivARB(fragmentWiggle, GL_OBJECT_COMPILE_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the compilation error
		GLint msglen;
		glGetObjectParameterivARB(fragmentWiggle, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(fragmentWiggle, msglen, NULL, msg);
		fprintf(stderr, "Error compiling fragment shader\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}
	
	// Attach the compiled snake shaders to the program object
	glAttachObjectARB(glslWiggle, vertexWiggle);
	glAttachObjectARB(glslWiggle, fragmentWiggle);
	
	// link the snake program
	glLinkProgramARB(glslWiggle);
	glGetObjectParameterivARB(glslWiggle, GL_OBJECT_LINK_STATUS_ARB, &status);
	if( GL_FALSE == status ) {
		// Get the InfoLog and print the linking error
		GLint msglen;
		glGetObjectParameterivARB(glslWiggle, GL_OBJECT_INFO_LOG_LENGTH_ARB, &msglen);
		GLcharARB* msg = (GLcharARB*)malloc(msglen);
		glGetInfoLogARB(glslWiggle, 1024, NULL, msg);
		fprintf(stderr, "Error linking GLSL program\n%s\n", msg);
		free(msg);
		exit(EXIT_FAILURE);
	}	
}

/**
*	Initializes the OpenGL state.  Loads the textures
*	and builds display lists.
*/
GLvoid Init(GLvoid)
{
	GLfloat  ambientLight[] = {0.1, 0.1, 0.1, 0.0};		// color of overall ambient light
	
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
	glEnable(GL_LINE_SMOOTH);
		glShadeModel(GL_SMOOTH);
	glEnable(GL_CULL_FACE);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	glEnable(GL_LIGHTING);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glEnable(GL_LIGHT0);
	
	glClearColor(0.2, 0.2, 0.2, 0.0);
	glClearDepth(1.0);
	
	// load all the textures and bind into texture objects
	glGenTextures(1, &duneTexture);		// standard texture objects
	glBindTexture(GL_TEXTURE_2D, duneTexture);
	image = read_texture("./textures/floor_normal_map.rgb", &width, &height, &components);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
	free(image);
	
	glGenTextures(1, &sandTexture);
	glBindTexture(GL_TEXTURE_2D, sandTexture);
	image = read_texture("./textures/sand.rgb", &width, &height, &components);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
	free(image);
	
	glGenTextures(1, &snakeColorTexture);
	glBindTexture(GL_TEXTURE_2D, snakeColorTexture);
	image = read_texture("./textures/snake_color_map.rgb", &width, &height, &components);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
	free(image);

	glGenTextures(1, &snakeBumpTexture);
	glBindTexture(GL_TEXTURE_2D, snakeBumpTexture);
	image = read_texture("./textures/snakeConvertedNormalMap3.rgb", &width, &height, &components);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
	free(image);

	// create the texture units from the texture objects
	glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, duneTexture);
	glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, sandTexture);
	glActiveTexture(GL_TEXTURE2);
		glBindTexture(GL_TEXTURE_2D, snakeColorTexture);
	glActiveTexture(GL_TEXTURE3);
		glBindTexture(GL_TEXTURE_2D, snakeBumpTexture);
	
	// create the display lists to hold two different gluCylinders
	// cylinderList = filled cylinder
	// cylinderList + 1 = wireframe cylinder
	cylinderList = glGenLists(2);
	
	GLUquadricObj *cylinder;
	cylinder = gluNewQuadric();
	
	gluQuadricDrawStyle(cylinder, GLU_FILL);
	gluQuadricTexture(cylinder, GL_TRUE);
	gluQuadricNormals(cylinder, GLU_SMOOTH);
	glNewList(cylinderList, GL_COMPILE);
		gluCylinder(cylinder, 1.0, 1.0, 10.0, 30.0, 30.0);
	glEndList();
	
	gluQuadricDrawStyle(cylinder, GLU_LINE);
	glNewList(cylinderList + 1, GL_COMPILE);
		gluCylinder(cylinder, 1.0, 1.0, 10.0, 30.0, 30.0);
	glEndList();
	
	// create the display lists to hold material properties
	// materialList = ground
	// materialList + 1 = snake
	materialList = glGenLists(2);
	GLfloat none[]			= {0.0, 0.0, 0.0, 1.0};
	GLfloat lowShiny[]		= {250.0};
	GLfloat yellow[]		= {0.6, 0.6, 0.2, 1.0};
	GLfloat green[]			= {0.4, 0.6, 0.4, 1.0};	
	GLfloat spec[]			= {1.0, 1.0, 1.0, 1.0};
	
	glNewList(materialList, GL_COMPILE);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, yellow);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, none);
	glEndList();
	
	glNewList(materialList + 1, GL_COMPILE);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, green);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, spec);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, lowShiny);
	glEndList();
}

/**
*	Initializes the texture uniform variables that will
*	be passed to the shader programs, including color and
*	normal map textures.
*/
GLvoid initUniformVars(GLvoid)
{	
	glUseProgramObjectARB(glslGroundBump);
		GLint bumpTextureHandle = glGetUniformLocationARB(glslGroundBump, "normalMap");
		GLint colorTextureHandle = glGetUniformLocationARB(glslGroundBump, "colorMap");
		
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, duneTexture);
		glUniform1iARB(bumpTextureHandle, 0);
		
		glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, sandTexture);
		glUniform1iARB(colorTextureHandle, 1);

	glUseProgramObjectARB(glslWiggle);
		GLint snakeColorTextureHandle = glGetUniformLocationARB(glslWiggle, "snakeColor");
		GLint snakeBumpTextureHandle = glGetUniformLocationARB(glslWiggle, "snakeBump");
		
		glActiveTexture(GL_TEXTURE3);
		glBindTexture(GL_TEXTURE_2D, snakeBumpTexture);
		glUniform1iARB(snakeBumpTextureHandle, 3);
		
		glActiveTexture(GL_TEXTURE2);
		glBindTexture(GL_TEXTURE_2D, snakeColorTexture);
		glUniform1iARB(snakeColorTextureHandle, 2);
	
	glUseProgramObjectARB(0);
}

/**
*	Draws the ground plane using GL_QUADS and the
*	appropriate material attributes.
*/
GLvoid drawGround(GLvoid)
{
	glCallList(materialList);

	glUseProgramObjectARB(glslGroundBump);
	glBegin(GL_QUADS);
	glNormal3f(0.0, 1.0, 0.0);
	glTexCoord2f(0.0, 0.0);			glVertex3f(-10.0, 0.0,-10.0);
	glTexCoord2f(0.0, 1.0);			glVertex3f(-10.0, 0.0, 10.0);
	glTexCoord2f(1.0, 1.0);			glVertex3f( 10.0, 0.0, 10.0);
	glTexCoord2f(1.0, 0.0);			glVertex3f( 10.0, 0.0,-10.0);
	glEnd();
	
	glUseProgramObjectARB(0);
}

/**
*	Draws the snake using the necessary display lists,
*	and initializes the uniform variables that control
*	the movement of the snake's sine function through time.
*
*	@param	mode	the snake drawing mode for fill/wireframe
*/
GLvoid drawSnake(GLint mode)
{
	glPushMatrix();
		glTranslatef(-5.0, 0.34, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glTranslatef(snakePosition[0], 0.0, snakePosition[2]);

		glUseProgramObjectARB(glslWiggle);

		GLint timeHandle = glGetUniformLocationARB(glslWiggle, "time");
			glUniform1fARB(timeHandle, myTime);

		GLint numberWigglesHandle = glGetUniformLocationARB(glslWiggle, "numberWiggles");
			glUniform1fARB(numberWigglesHandle, myNumberWiggles);

		GLint speedHandle = glGetUniformLocationARB(glslWiggle, "speed");
			glUniform1fARB(speedHandle, mySpeed);

		GLint lightPosHandle = glGetUniformLocationARB(glslWiggle, "lightPosition");
			glUniform3fvARB(lightPosHandle, 1, lightPos);

		glCallList(materialList + 1);
		switch (mode)
		{
			case SNAKE_WIRE:
				glCallList(cylinderList + 1);
				break;
			case SNAKE_SOLID:
				glCallList(cylinderList);
				break;
		}

		glUseProgramObjectARB(0);

	glPopMatrix();
}

/**
*	Sets up the projection and modelview matricies, positions
*	and colors the light, and draws the geometry.
*/
void Display(void)
{	
	diffuseLight[0] = diffuseLight[1] = diffuseLight[2] = lightIntensity;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	gluPerspective(70.0, 1.0, 0.1, 200.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	lightPos[0] = lightPositionXZ[0];
	lightPos[2] = lightPositionXZ[1];
	glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
		
	gluLookAt(eye[0], eye[1], eye[2], lookat[0], lookat[1], lookat[2], 0, 1, 0);
	
	drawGround();
	drawSnake(snakeMode);

	glutSwapBuffers();
}

/**
*	The GLUI callback for GLUI buttons, etc.
 *	Resets the camera.
 *
 *	@param	control		the GLUI state to control
 */
void glui_cb(int control)
{
	switch(control)
	{
		case 1:
			eye[0] = 0;
			eye[1] = 5;
			eye[2] = -10;
			lookat[0] = 0;
			lookat[1] = 0;
			lookat[2] = 0;
			break;
	}
	
	glutPostRedisplay();
}

/**
*	The main function.  Sets up the GLUT callback functions,
 *	and builds the GLUI GUI interface.
 */
int main(int argc, char** argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL);
    glutInitWindowSize(WINDOW_WIDTH + viewX, WINDOW_HEIGHT); 
    glutInitWindowPosition (100, 100);
    
	main_window = glutCreateWindow ("Snake!");
	glewInit();
	Init();
	
    glutReshapeFunc(myGlutReshape);
	
    glutDisplayFunc(Display); 
	glutMouseFunc(myGlutMouse);
	glutMotionFunc(myGlutMotion);
	glutSpecialFunc(processArrowKeys);
	glutIdleFunc(myGlutIdle);
	
	createShaders();
	initUniformVars();
	
	glui = GLUI_Master.create_glui_subwindow(main_window, GLUI_SUBWINDOW_LEFT);
	glui->add_button("Quit", 0, (GLUI_Update_CB)exit);	
	glui->add_button("Reset Camera", 1, glui_cb);
	
	lightRollout = glui->add_rollout("Light Control");
	lightPanel = glui->add_panel_to_panel(lightRollout, "", GLUI_PANEL_NONE);
	lightSpin = glui->add_spinner_to_panel(lightRollout, "Light Intensity", GLUI_SPINNER_FLOAT, &lightIntensity);
	lightSpin->set_float_limits(0.5, 5.0);
	
	lightTransXY = glui->add_translation_to_panel(lightPanel, "Light XZ Position", GLUI_TRANSLATION_XY, lightPositionXZ);
	lightTransXY->scale_factor = 0.1;

	snakeRollout = glui->add_rollout("Snake Control");
	snakePanel = glui->add_panel_to_panel(snakeRollout, "", GLUI_PANEL_NONE);
	snakeRadio = glui->add_radiogroup_to_panel(snakePanel, &snakeMode);
	glui->add_radiobutton_to_group(snakeRadio, "Wireframe");
	glui->add_radiobutton_to_group(snakeRadio, "Solid Surface");
	speedSpin = glui->add_spinner_to_panel(snakePanel, "Speed", GLUI_SPINNER_FLOAT, &mySpeed);
	speedSpin->set_float_limits(0.5, 5.0);
	numberSpin = glui->add_spinner_to_panel(snakePanel, "Number of Wiggles", GLUI_SPINNER_FLOAT, &myNumberWiggles);
	numberSpin->set_float_limits(0.5, 4.5);
	
	eye[0] = 0;
	eye[1] = 5;
	eye[2] = -10;
	lookat[0] = 0;
	lookat[1] = 0;
	lookat[2] = 0;
	
    glutMainLoop();
    
    return 0;
}

