/**
*	Positions each vertex according to a sine function
*	to simulate wiggling, tapers the cylinder into
*	snake form (see report), and sets up tangent space
*	to be passed to the fragment shader for bump mapping
*	and specular lighting.
*/

	uniform float time;				// the time counter for animation
	uniform float numberWiggles;	// number of wiggles, (i.e. 1.0 = ), 2.0 = S)
	uniform float speed;			// the speed of the wiggles
	uniform vec3 lightPosition;		// the position of the light

	varying vec3 lightVec;			// OUT - tangent space light vector
	varying vec3 eyeVec;			// OUT - tangent space eye vector
	varying vec2 texCoord;			// OUT - cylinder texture coordinate
	attribute vec3 tangent;			// tangent vector passed into vertex program

void main()
{
	// BEGIN - Wiggle transformation code
	vec4 position = vec4(gl_Vertex);
	float radius;

	// I use a quadratic function rather than a sine function to ensure a smooth taper
	radius = (position.z < 1.0) ? sqrt(position.z / 1.0): sqrt((10.0 - position.z) / 1.0);
	position.xy *= (position.z < 1.0 || position.z > 9.0) ? radius : 1.0;
	position.xy = position.xy * 0.33;
	
	// wiggle the snake along a centerline sine function
	position.x += sin(position.z * (numberWiggles / 2.0) + time * 0.1 * speed);

	gl_Position = gl_ModelViewProjectionMatrix * position;
	// END - Wiggle transformation code
	
	// BEGIN - Setup Tangent Space
	eyeVec = vec3(gl_ModelViewMatrix * gl_Vertex);
	gl_TexCoord[0] = gl_MultiTexCoord0;

	// build T,B,N verticies
	vec3 N = normalize(gl_NormalMatrix * gl_Normal);
	vec3 T = normalize(gl_NormalMatrix * tangent);
	vec3 B = cross(T, N);
	
	vec3 V;		// temp working vector
	
	// tangent space light vector is simply
	// negation of standard light vector
	V.x = -lightPosition.x;
	V.y = -lightPosition.y;
	V.z = -lightPosition.z;
	lightVec = normalize(V);

	// calculate tangent space eye vector
	V.x = dot(T, eyeVec);
	V.y = dot(B, eyeVec);
	V.z = dot(N, eyeVec);
	eyeVec = normalize(V);	
	// END - Tangent Space
	
	// increases the texture use for more snake-like appearance
	texCoord = vec2(gl_MultiTexCoord0);
	texCoord.t = texCoord.t * 4.0;

}