/**
*	Using each fragments interpolated tanget space
*	light and eye vextor, perturbs the sampled normals
*	to create a bumpmapped image, and calculates
*	specular lighting using the Blinn/Phong half
*	angle model of lighting.  All lighting is done
*	in tangent space as well.  See the report for
*	the full lighting equation.
*/

	uniform sampler2D snakeBump;		// the normal texture
	uniform sampler2D snakeColor;		// the color texture
	
	varying vec3 lightVec;				// IN - tangent space light vector
	varying vec3 eyeVec;				// IN - tangent space eye vector
	varying vec2 texCoord;				// interpolated texture coordinate
		
void main()
{
    vec4 color;		// working color term
	vec4 specular;	// working specular term
	
	// perterb each interpolated normal according to the normal map
	vec3 normalTex = texture2D(snakeBump, texCoord).xyz;
	normalTex = (normalTex * 2.0) - 1.0;
	vec3 newNormal = normalize(normalTex);
	
	float NdotL = max(dot(newNormal, lightVec), 0.0);
	
	// calculate the diffuse term
	vec4 diffuse = gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse;
	
	// calculate the ambient and global ambient terms
	vec4 ambient = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
	vec4 globalAmbient = gl_LightModel.ambient * gl_FrontMaterial.ambient;
	
	// interpolate the fragment's color from the sampled texture
    vec4 textureColor = texture2D(snakeColor, texCoord);
	
	// using the half angle vector, compute the speculat term
	vec3 halfVec = normalize(lightVec + eyeVec);
	if (NdotL > 0.0)
	{
		float NdotHV = max(dot(newNormal, halfVec), 0.0);
		specular = gl_FrontMaterial.specular * gl_LightSource[0].specular * pow(NdotHV, gl_FrontMaterial.shininess);
	}
	
	gl_FragColor = min(textureColor * NdotL * diffuse + ambient + globalAmbient + specular, vec4(1.0));	
}